use std::{
    net::{TcpListener, TcpStream},
    io::{prelude::*, BufReader},
    fs,
    thread,
    time::Duration,
};

use rust_webserver::ThreadPool;

fn parse_request(stream: &TcpStream) -> Vec<String>{
    let buf_reader = BufReader::new(stream);

    buf_reader
        .lines()
        .map(|result| result.unwrap())
        .take_while(|line| !line.is_empty())
        .collect()
}

fn generate_reposne(request: &Vec<String>) -> String {
    let request_header = (*(request.first().unwrap())).clone();

    let (status, file) = 
        match &request_header[..] {
            "GET / HTTP/1.1" => ("HTTP/1.1 200 OK", "html/index.html"),
            "GET /sleep HTTP/1.1" => {
                thread::sleep(Duration::from_secs(5));
                ("HTTP/1.1 200 OK", "html/slow.html")
            },
            _ => ("HTTP/1.1 404 NOT FOUND", "html/404.html")
        };

    let contents = fs::read_to_string(file).unwrap();
    let len = contents.len();
    format!("{status}\r\nContent-Length: {len}\r\n\r\n{contents}")
}

fn handle_connection(mut stream: TcpStream) {
    let request = parse_request(&stream);
    let response = generate_reposne(&request);

    stream.write_all(response.as_bytes()).unwrap();
}

fn main() {
    let port = 7878;
    let connections = 3;
    let listener = TcpListener::bind(format!("127.0.0.1:{}", port)).unwrap();

    let pool = ThreadPool::new(10).unwrap();

    println!("Listening for {} connnections on port {}", connections, port);
    for stream in listener.incoming().take(connections) {
        let stream = stream.unwrap();
        
        pool.execute(|| {
            handle_connection(stream);
        });

    }
}
